from .controllers.google_api import GoogleAPIAuth, get_comments
from .controllers.nlp import OsetiAnalyzer

from typing import List, Dict

import pandas as pd

class CommentAnalyzer:
    """YouTube comment analyzer
    Args:
        video_id (str): YouTube video ID of the video to get comments from
        api_token_file_path (str): Google API token file path (json file)
    """

    def __init__(self, video_id: str, api_token_file_path: str):
        self.video_id = video_id
        self.api_token_obj = GoogleAPIAuth(api_token_file_path).get_authenticated_service()

    def get_comments(self) -> List[Dict]:
        """get comments from video

        Returns:
            List[Dict]: comments
        """
        return get_comments(self.api_token_obj, self.video_id)

    def get_analyzed_comments(self, comments: List[Dict]) -> List[Dict]:
        """add oseti score

        Args:
            comments (List[Dict]): comments in list-in-dict format

        Returns:
            List[Dict]: comments with oseti score
        """
        oa = OsetiAnalyzer()
        for comment in comments:
            try:
                comment["oseti_score"] = oa.analyze(comment["snippet"]["topLevelComment"]["snippet"]["textOriginal"])
            except:
                comment["oseti_score"] = 0
        return comments

    def get_summarized_comments(self, comments: List[Dict], summarized_in: str = "W") -> pd.DataFrame:
        """get summarized comments grouped by datetime (default week)

        Args:
            comments (List[Dict]): comments
            summarized_in (str, optional): how to group by comments. Please refer to pandas resample documentation. Defaults to "W".

        Returns:
            pd.DataFrame: summarized comments grouped by datetime (default week)
        """
        df = pd.json_normalize(comments)
        df['snippet.topLevelComment.snippet.publishedAt'] = pd.to_datetime(df['snippet.topLevelComment.snippet.publishedAt'])
        df = df.set_index('snippet.topLevelComment.snippet.publishedAt')
        return df.resample(summarized_in).sum()


import os
import pickle
import google_auth_oauthlib.flow
import googleapiclient.discovery
import googleapiclient.errors

class GoogleAPIAuth:
    """Google API OAuth2.0 authentication class
    """

    def __init__(self, client_secret_json):
        self.CLIENT_SECRETS_FILE = client_secret_json
        self.SCOPES = ['https://www.googleapis.com/auth/youtubepartner', 'https://www.googleapis.com/auth/youtube', 'https://www.googleapis.com/auth/youtube.force-ssl']
        self.API_SERVICE_NAME = 'youtube'
        self.API_VERSION = 'v3'

    def get_authenticated_service(self):
        """Initialize Google Auth service

        Returns:
            googleapiclient object
        """
        if os.path.exists("CREDENTIALS_PICKLE_FILE"):
            with open("CREDENTIALS_PICKLE_FILE", 'rb') as f:
                credentials = pickle.load(f)
        else:
            flow = google_auth_oauthlib.flow.InstalledAppFlow.from_client_secrets_file(self.CLIENT_SECRETS_FILE, self.SCOPES)
            credentials = flow.run_console()
            with open("CREDENTIALS_PICKLE_FILE", 'wb') as f:
                pickle.dump(credentials, f)
        return googleapiclient.discovery.build(self.API_SERVICE_NAME, self.API_VERSION, credentials=credentials)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ytcomment_trends',
 'ytcomment_trends.controllers.google_api',
 'ytcomment_trends.controllers.nlp']

package_data = \
{'': ['*']}

install_requires = \
['google-api-python-client>=2.36.0,<3.0.0',
 'google-auth-httplib2>=0.1.0,<0.2.0',
 'google-auth-oauthlib>=0.4.6,<0.5.0',
 'matplotlib>=3.5.1,<4.0.0',
 'oseti>=0.2,<0.3',
 'pandas>=1.3.5,<2.0.0']

entry_points = \
{'console_scripts': ['ytcomment_trends = '
                     'ytcomment_trends.entrypoint:entrypoint']}

setup_kwargs = {
    'name': 'ytcomment-trends',
    'version': '0.1.2',
    'description': 'YouTube comment trends analysis tool using oseti',
    'long_description': '# ytcomment_trends\n\n## Dependencies\n\nBefore install this library, you need to install mecab for NLP.\n\nFor macOS, run this command to install mecab and ipadic dictionary. For other OS, please follow the instructions from mecab official documentation.\n\n```\nbrew install mecab mecab-ipadic\n```\n\n## How to use\n\n### Get YouTube API Client Secret\n\nPlease refer to [Google\'s official documentation](https://developers.google.com/youtube/registering_an_application) for getting API keys. Make sure you create credentials with OAuth 2.0 Clients with type of "Desktop" app. Also, make sure you download JSON file to your local directory.\n\n### Run command\n\nInstall this library with the following command:\n\n```\npip install ytcomment_trends\n```\n\nIf you are using virtual environment, please use the package manager of the virtual environment (e.g., `pipenv install`, `poetry add`).\n\nAfter installation, run this command to analyze video. Make sure to check credentials JSON file path and video ID.\n\n```\nytcomment_trends -t "./client_secret.json" -v pR2E2OatMTQ\n```\n\nIf you are not sure about the arguments, run following command to check.\n\n```\nytcomment_trends -h\n```',
    'author': 'Kyosuke Miyamura',
    'author_email': 'ask@386.jp',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://www.386.jp/works/ytcomment_trends',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



import oseti

class OsetiAnalyzer:
    """oseti Japanese text analyzer
    """

    def __init__(self):
        self.analyzer = oseti.Analyzer()

    def analyze(self, text: str) -> float:
        """get negative / positive value

        Args:
            text (str): text to analyze

        Returns:
            float: negative / positive value from -1 to 1
        """
        result = self.analyzer.analyze(text)
        return sum(result) / len(result)


import argparse
import matplotlib.pyplot as plt
from .main import CommentAnalyzer

def entrypoint():
    """Entrypoint for the ytcomment_trends package
    """
    parser = argparse.ArgumentParser(prog='ytcomment_trends', usage='ytcomment_trends -v pR2E2OatMTQ -t "./client_secret.json"', description='ytcomment_trends: YouTube comment trends analysis tool using oseti')
    parser.add_argument('-v', '--video_id', help='YouTube video id', type=str, required=True)
    parser.add_argument('-t', '--token', help='YouTube API token (json) file path', type=str, required=True)
    args = parser.parse_args()

    ca = CommentAnalyzer(args.video_id, args.token)
    ca_comments = ca.get_comments()
    ca_analyzed = ca.get_analyzed_comments(ca_comments)
    ca_summarized = ca.get_summarized_comments(ca_analyzed)

    fig, ax1 = plt.subplots()
    t = list(ca_summarized.index.values)

    color = 'tab:red'
    ax1.set_xlabel('datetime comment posted')
    ax1.set_ylabel('number of comments', color=color)
    ax1.plot(t, list(ca_summarized['snippet.isPublic']), color=color)
    ax1.tick_params(axis='y', labelcolor=color)

    ax2 = ax1.twinx()

    oseti_scores = []
    for s, n in zip(list(ca_summarized['oseti_score']), list(ca_summarized['snippet.isPublic'])):
        if n > 0:
            oseti_scores.append(s)
        else:
            oseti_scores.append(0)

    color = 'tab:blue'
    ax2.set_ylabel('negative / positive', color=color)
    ax2.plot(t, oseti_scores, color=color)
    ax2.tick_params(axis='y', labelcolor=color)

    plt.ylim(-1.2, 1.2)
    plt.title("YouTube Video Comment Trends for " + args.video_id)
    plt.grid(True)

    fig.tight_layout()
    plt.show()

if __name__ == "__main__":
    entrypoint()


from typing import List, Dict

def get_comments(api_service, video_id: str, nextPageToken=None, output=[]) -> List[Dict]:
    """get YouTube comments for video

    Args:
        api_service (object): Google API Authentication Service
        video_id (str): YouTube video ID of the video to get comments from

    Returns:
        List[Dict]: All comments in list-in-dict format
    """
    request = api_service.commentThreads().list(
        part="snippet",
        videoId=video_id,
        pageToken=nextPageToken
    ).execute()
    for info in request["items"]:
        output.append(info)
    try:
        nextPageToken = request["nextPageToken"]
    except:
        return output
    else:
        return get_comments(api_service, video_id, nextPageToken, output)